@@ -1,7 +1,6 @@
 """AMQP Channels
 """
 import logging
-from collections import defaultdict
 from queue import Queue
 
 from .concurrency import synchronized
@@ -18,15 +17,11 @@ log = logging.getLogger('amqpy')
 
 
 class Channel(AbstractChannel):
-    """Work with channels
-
-    The channel class provides methods for a client to establish a virtual connection - a channel - to a server and for
+    """
+    The channel class provides methods for a client to establish a virtual connection (a channel) to a server and for
     both peers to operate the virtual connection thereafter.
     """
 
-    # TODO: finish cleaning up the documentation in this module
-    # TODO: add :raise: and :return: directives in each docstring
-
     def __init__(self, connection, channel_id=None, auto_decode=True):
         """Create a channel bound to a connection and using the specified numeric channel_id, and open on the server
 
@@ -63,7 +58,6 @@ class Channel(AbstractChannel):
         self.cancel_callbacks = {}
 
         self.auto_decode = auto_decode  # auto decode received messages
-        self.events = defaultdict(set)  # TODO: find out what this is used for
 
         # set of consumers that have opted for `no_ack` delivery (server will not expect an ack for delivered messages)
         self.no_ack_consumers = set()
@@ -86,7 +80,6 @@ class Channel(AbstractChannel):
             connection._avail_channel_ids.append(channel_id)
         self.callbacks.clear()
         self.cancel_callbacks.clear()
-        self.events.clear()
         self.no_ack_consumers.clear()
 
     def _do_revive(self):
@@ -122,13 +115,15 @@ class Channel(AbstractChannel):
         finally:
             self.connection = None
 
-    def _close(self, method):
+    def _cb_close(self, method):
         """Respond to a channel close
 
         This method indicates that the sender (server) wants to close the channel. This may be due to internal
         conditions (e.g. a forced shut-down) or due to an error handling a specific method, i.e. an exception. When a
         close is due to an exception, the sender provides the class and method id of the method which caused the
         exception.
+
+        This method sends a "close-ok" to the server, then re-opens the channel.
         """
         args = method.args
         reply_code = args.read_short()
@@ -137,12 +132,16 @@ class Channel(AbstractChannel):
         method_id = args.read_short()
 
         self._send_method(Method(spec.Channel.CloseOk))
+        self.is_open = False
+
+        # re-open the channel
         self._do_revive()
 
+        # get information about the method which caused the server to close the channel
         method_type = method_t(class_id, method_id)
         raise error_for_code(reply_code, reply_text, method_type, ChannelError, self.channel_id)
 
-    def _close_ok(self, method):
+    def _cb_close_ok(self, method):
         """Confirm a channel close
 
         This method confirms a Channel.Close method and tells the recipient that it is safe to release resources for the
@@ -156,7 +155,7 @@ class Channel(AbstractChannel):
         """Enable/disable flow from peer
 
         This method asks the peer to pause or restart the flow of content data. This is a simple flow-control mechanism
-        that a peer can use to avoid oveflowing its queues or otherwise finding itself receiving more messages than it
+        that a peer can use to avoid overflowing its queues or otherwise finding itself receiving more messages than it
         can process. Note that this method is not intended for window control.  The peer that receives a request to stop
         sending content should finish sending the current content, if any, and then wait until it receives a Flow
         restart method.
@@ -169,7 +168,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Channel.Flow, args))
         return self.wait(allowed_methods=[spec.Channel.FlowOk])
 
-    def _flow(self, method):
+    def _cb_flow(self, method):
         """Enable/disable flow from peer
 
         This method asks the peer to pause or restart the flow of content data. This is a simple flow-control mechanism
@@ -194,7 +193,7 @@ class Channel(AbstractChannel):
         args.write_bit(active)
         self._send_method(Method(spec.Channel.FlowOk, args))
 
-    def _flow_ok(self, method):
+    def _cb_flow_ok(self, method):
         """Confirm a flow method
 
         Confirms to the peer that a flow command was received and processed.
@@ -215,7 +214,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Channel.Open, args))
         return self.wait(allowed_methods=[spec.Channel.OpenOk])
 
-    def _open_ok(self, method):
+    def _cb_open_ok(self, method):
         """Signal that the channel is ready
 
         The server sends this method to signal to the client that this channel is ready for use.
@@ -225,7 +224,7 @@ class Channel(AbstractChannel):
         log.debug('Channel open')
 
     @synchronized('lock')
-    def exchange_declare(self, exch_name, exch_type, passive=False, durable=False, auto_delete=True, nowait=False,
+    def exchange_declare(self, exchange, exch_type, passive=False, durable=False, auto_delete=True, nowait=False,
                          arguments=None):
         """Declare exchange, create if needed
 
@@ -239,7 +238,7 @@ class Channel(AbstractChannel):
           exception.
         * `arguments` is ignored if passive is True.
 
-        :param str exch_name: exchange name
+        :param str exchange: exchange name
         :param str exch_type: exchange type (direct, fanout, etc.)
         :param bool passive: do not create exchange; client can use this to check whether an exchange exists
         :param bool durable: mark exchange as durable (remain active after server restarts)
@@ -253,7 +252,7 @@ class Channel(AbstractChannel):
         arguments = arguments or {}
         args = AMQPWriter()
         args.write_short(0)  # reserved-1
-        args.write_shortstr(exch_name)  # exchange name
+        args.write_shortstr(exchange)  # exchange name
         args.write_shortstr(exch_type)  # exchange type
         args.write_bit(passive)  # passive
         args.write_bit(durable)  # durable
@@ -266,7 +265,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Exchange.DeclareOk])
 
-    def _exchange_declare_ok(self, method):
+    def _cb_exchange_declare_ok(self, method):
         """Confirms an exchange declaration
 
         The server sends this method to confirm a Declare method and confirms the name of the exchange, essential for
@@ -275,7 +274,7 @@ class Channel(AbstractChannel):
         pass
 
     @synchronized('lock')
-    def exchange_delete(self, exch_name, if_unused=False, nowait=False):
+    def exchange_delete(self, exchange, if_unused=False, nowait=False):
         """Delete an exchange
 
         This method deletes an exchange.
@@ -284,16 +283,16 @@ class Channel(AbstractChannel):
           all queue bindings on the exchange are cancelled.
         * If `if_unused` is set, and the exchange has queue bindings, the server must raise a channel exception.
 
-        :param str exch_name: exchange name
+        :param str exchange: exchange name
         :param bool if_unused: delete only if unused (has no queue bindings)
         :param bool nowait: if set, the server will not respond to the method and the client should not wait for a reply
-        :raise NotFound: if exchange with `exch_name` does not exist
+        :raise NotFound: if exchange with `exchange` does not exist
         :raise PreconditionFailed: if attempting to delete a queue with bindings and `if_unused` is set
         :return: None
         """
         args = AMQPWriter()
         args.write_short(0)
-        args.write_shortstr(exch_name)
+        args.write_shortstr(exchange)
         args.write_bit(if_unused)
         args.write_bit(nowait)
         self._send_method(Method(spec.Exchange.Delete, args))
@@ -301,7 +300,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Exchange.DeleteOk])
 
-    def _exchange_delete_ok(self, method):
+    def _cb_exchange_delete_ok(self, method):
         """Confirm deletion of an exchange
 
         The server sends this method to confirm that the deletion of an exchange was successful.
@@ -366,14 +365,14 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Exchange.UnbindOk])
 
-    def _exchange_bind_ok(self, method):
+    def _cb_exchange_bind_ok(self, method):
         """Confirm bind successful
 
         The server sends this method to confirm that the bind was successful.
         """
         pass
 
-    def _exchange_unbind_ok(self, method):
+    def _cb_exchange_unbind_ok(self, method):
         """Confirm unbind successful
 
         The server sends this method to confirm that the unbind was successful.
@@ -381,7 +380,7 @@ class Channel(AbstractChannel):
         pass
 
     @synchronized('lock')
-    def queue_bind(self, queue_name, exch_name='', routing_key='', nowait=False, arguments=None):
+    def queue_bind(self, queue, exchange='', routing_key='', nowait=False, arguments=None):
         """Bind queue to an exchange
 
         This method binds a queue to an exchange. Until a queue is bound it will not receive any messages. In a classic
@@ -395,13 +394,13 @@ class Channel(AbstractChannel):
         * The server should support at least 4 bindings per queue, and ideally, impose no limit except as defined by
           available resources.
 
-        * If the client did not previously declare a queue, and the `queue_name` is empty, the server must raise a
+        * If the client did not previously declare a queue, and the `queue` is empty, the server must raise a
           connection exception with reply code 530 (not allowed).
-        * If `queue_name` does not exist, the server must raise a channel exception with reply code 404 (not found).
-        * If `exch_name` does not exist, the server must raise a channel exception with reply code 404 (not found).
+        * If `queue` does not exist, the server must raise a channel exception with reply code 404 (not found).
+        * If `exchange` does not exist, the server must raise a channel exception with reply code 404 (not found).
 
-        :param str queue_name: name of queue to bind; blank refers to the last declared queue for this channel
-        :param str exch_name: name of exchange to bind to
+        :param str queue: name of queue to bind; blank refers to the last declared queue for this channel
+        :param str exchange: name of exchange to bind to
         :param str routing_key: routing key for the binding
         :param bool nowait: if set, the server will not respond to the method and the client should not wait for a reply
         :param dict arguments: binding arguments, specific to the exchange class
@@ -409,8 +408,8 @@ class Channel(AbstractChannel):
         arguments = {} if arguments is None else arguments
         args = AMQPWriter()
         args.write_short(0)
-        args.write_shortstr(queue_name)
-        args.write_shortstr(exch_name)
+        args.write_shortstr(queue)
+        args.write_shortstr(exchange)
         args.write_shortstr(routing_key)
         args.write_bit(nowait)
         args.write_table(arguments)
@@ -419,7 +418,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Queue.BindOk])
 
-    def _queue_bind_ok(self, method):
+    def _cb_queue_bind_ok(self, method):
         """Confirm bind successful
 
         The server sends this method to confirm that the bind was successful.
@@ -427,7 +426,7 @@ class Channel(AbstractChannel):
         pass
 
     @synchronized('lock')
-    def queue_unbind(self, queue_name, exch_name, routing_key='', nowait=False, arguments=None):
+    def queue_unbind(self, queue, exchange, routing_key='', nowait=False, arguments=None):
         """Unbind a queue from an exchange
 
         This method unbinds a queue from an exchange.
@@ -436,16 +435,16 @@ class Channel(AbstractChannel):
         * The client must not attempt to unbind a queue that does not exist.
         * The client must not attempt to unbind a queue from an exchange that does not exist.
 
-        :param str queue_name: name of queue to unbind, leave blank to refer to the last declared queue on this channel
-        :param str exch_name: name of exchange to unbind, leave blank to refer to default exchange
+        :param str queue: name of queue to unbind, leave blank to refer to the last declared queue on this channel
+        :param str exchange: name of exchange to unbind, leave blank to refer to default exchange
         :param str routing_key: routing key of binding
         :param dict arguments: binding arguments, specific to the exchange class
         """
         arguments = {} if arguments is None else arguments
         args = AMQPWriter()
         args.write_short(0)
-        args.write_shortstr(queue_name)
-        args.write_shortstr(exch_name)
+        args.write_shortstr(queue)
+        args.write_shortstr(exchange)
         args.write_shortstr(routing_key)
         # args.write_bit(nowait)
         args.write_table(arguments)
@@ -454,7 +453,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Queue.UnbindOk])
 
-    def _queue_unbind_ok(self, method):
+    def _cb_queue_unbind_ok(self, method):
         """Confirm unbind successful
 
         This method confirms that the unbind was successful.
@@ -493,7 +492,7 @@ class Channel(AbstractChannel):
         :raise NotFound: if `passive` is enabled and the queue does not exist
         :raise AccessRefused: if an attempt is made to declare a queue with a reserved name
         :raise ResourceLocked: if an attempt is made to access an exclusive queue declared by another open connection
-        :return: tuple(queue_name, message_count, consumer_count)
+        :return: tuple(queue, message_count, consumer_count)
         :rtype: tuple(str, int, int)
         """
         arguments = {} if arguments is None else arguments
@@ -511,7 +510,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Queue.DeclareOk])
 
-    def _queue_declare_ok(self, method):
+    def _cb_queue_declare_ok(self, method):
         """Confirms a queue definition
 
         This method confirms a Declare method and confirms the name of the queue, essential for automatically-named
@@ -540,24 +539,24 @@ class Channel(AbstractChannel):
         return queue_declare_ok_t(args.read_shortstr(), args.read_long(), args.read_long())
 
     @synchronized('lock')
-    def queue_delete(self, queue_name='', if_unused=False, if_empty=False, nowait=False):
+    def queue_delete(self, queue='', if_unused=False, if_empty=False, nowait=False):
         """Delete a queue
 
         This method deletes a queue. When a queue is deleted any pending messages are sent to a dead-letter queue if
         this is defined in the server configuration, and all consumers on the queue are cancelled.
 
-        :param str queue_name: name of queue to delete, empty string refers to last declared queue on this channel
+        :param str queue: name of queue to delete, empty string refers to last declared queue on this channel
         :param bool if_unused: delete only if unused (has no consumers); raise a channel exception otherwise
         :param bool if_empty: delete only if empty; raise a channel exception otherwise
         :param bool nowait: if set, the server will not respond to the method and the client should not wait for a reply
-        :raise NotFound: if `queue_name` does not exist
+        :raise NotFound: if `queue` does not exist
         :raise PreconditionFailed: if `if_unused` or `if_empty` conditions are not met
         :return: number of messages deleted
         :rtype: int
         """
         args = AMQPWriter()
         args.write_short(0)
-        args.write_shortstr(queue_name)
+        args.write_shortstr(queue)
         args.write_bit(if_unused)
         args.write_bit(if_empty)
         args.write_bit(nowait)
@@ -566,7 +565,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Queue.DeleteOk])
 
-    def _queue_delete_ok(self, method):
+    def _cb_queue_delete_ok(self, method):
         """Confirm deletion of a queue
 
         This method confirms the deletion of a queue.
@@ -606,7 +605,7 @@ class Channel(AbstractChannel):
         if not nowait:
             return self.wait(allowed_methods=[spec.Queue.PurgeOk])
 
-    def _queue_purge_ok(self, method):
+    def _cb_queue_purge_ok(self, method):
         """Confirms a queue purge
 
         This method confirms the purge of a queue.
@@ -662,7 +661,7 @@ class Channel(AbstractChannel):
             self._send_method(Method(spec.Basic.Cancel, args))
             return self.wait(allowed_methods=[spec.Basic.CancelOk])
 
-    def _basic_cancel_notify(self, method):
+    def _cb_basic_cancel_notify(self, method):
         """Consumer cancelled by server.
 
         Most likely the queue was deleted.
@@ -675,7 +674,7 @@ class Channel(AbstractChannel):
         else:
             raise ConsumerCancelled(consumer_tag, spec.Basic.Cancel)
 
-    def _basic_cancel_ok(self, method):
+    def _cb_basic_cancel_ok(self, method):
         """Confirm a cancelled consumer
 
         This method confirms that the cancellation was completed.
@@ -758,7 +757,7 @@ class Channel(AbstractChannel):
 
         return consumer_tag
 
-    def _basic_consume_ok(self, method):
+    def _cb_basic_consume_ok(self, method):
         """Confirm a new consumer
 
         The server provides the client with a consumer tag, which is used by the client for methods called on the
@@ -773,7 +772,7 @@ class Channel(AbstractChannel):
         args = method.args
         return args.read_shortstr()
 
-    def _basic_deliver(self, method):
+    def _cb_basic_deliver(self, method):
         """Notify the client of a consumer message
 
         This method delivers a message to the client, via a consumer. In the asynchronous message delivery model, the
@@ -877,7 +876,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Basic.Get, args))
         return self.wait(allowed_methods=[spec.Basic.GetOk, spec.Basic.GetEmpty])
 
-    def _basic_get_empty(self, method):
+    def _cb_basic_get_empty(self, method):
         """Indicate no messages available
 
         This method tells the client that the queue has no messages
@@ -886,7 +885,7 @@ class Channel(AbstractChannel):
         args = method.args
         args.read_shortstr()
 
-    def _basic_get_ok(self, method):
+    def _cb_basic_get_ok(self, method):
         """Provide client with a message
 
         This method delivers a message to the client following a get method.  A message delivered by 'get-ok' must be
@@ -1042,7 +1041,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Basic.Qos, args))
         return self.wait(allowed_methods=[spec.Basic.QosOk])
 
-    def _basic_qos_ok(self, method):
+    def _cb_basic_qos_ok(self, method):
         """Confirm the requested qos
 
         This method tells the client that the requested QoS levels could be handled by the server.  The requested QoS
@@ -1084,7 +1083,7 @@ class Channel(AbstractChannel):
         args.write_bit(requeue)
         self._send_method(Method(spec.Basic.RecoverAsync, args))
 
-    def _basic_recover_ok(self, method):
+    def _cb_basic_recover_ok(self, method):
         """In 0-9-1 the deprecated recover solicits a response
         """
         pass
@@ -1117,7 +1116,7 @@ class Channel(AbstractChannel):
         args.write_bit(requeue)
         self._send_method(Method(spec.Basic.Reject, args))
 
-    def _basic_return(self, method):
+    def _cb_basic_return(self, method):
         """Return a failed message
 
         This method returns an undeliverable message that was published with the `immediate` flag set, or an unroutable
@@ -1144,7 +1143,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Tx.Commit))
         return self.wait(allowed_methods=[spec.Tx.CommitOk])
 
-    def _tx_commit_ok(self, method):
+    def _cb_tx_commit_ok(self, method):
         """Confirm a successful commit
 
         This method confirms to the client that the commit succeeded. Note that if a commit fails, the server raises a
@@ -1162,7 +1161,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Tx.Rollback))
         return self.wait(allowed_methods=[spec.Tx.RollbackOk])
 
-    def _tx_rollback_ok(self, method):
+    def _cb_tx_rollback_ok(self, method):
         """Confirm a successful rollback
 
         This method confirms to the client that the rollback succeeded. Note that if an rollback fails, the server
@@ -1180,7 +1179,7 @@ class Channel(AbstractChannel):
         self._send_method(Method(spec.Tx.Select))
         return self.wait(allowed_methods=[spec.Tx.SelectOk])
 
-    def _tx_select_ok(self, method):
+    def _cb_tx_select_ok(self, method):
         """Confirm transaction mode
 
         This method confirms to the client that the channel was successfully set to use standard transactions.
@@ -1203,49 +1202,47 @@ class Channel(AbstractChannel):
             self.wait(allowed_methods=[spec.Confirm.SelectOk])
         self.publisher_ack_enabled = True
 
-    def _confirm_select_ok(self, method):
+    def _cb_confirm_select_ok(self, method):
         """With this method, the broker confirms to the client that the channel is now using publisher confirms
         """
         pass
 
-    def _basic_ack_recv(self, method):
+    def _cb_basic_ack_recv(self, method):
         """Callback for receiving a `spec.Basic.Ack`
 
         This will be called when the server acknowledges a published message (RabbitMQ extension).
         """
-        args = method.args
-        delivery_tag = args.read_longlong()
-        multiple = args.read_bit()
-        for callback in self.events['basic_ack']:
-            callback(delivery_tag, multiple)
+        # args = method.args
+        # delivery_tag = args.read_longlong()
+        # multiple = args.read_bit()
 
     METHOD_MAP = {
-        spec.Channel.OpenOk: _open_ok,
-        spec.Channel.Flow: _flow,
-        spec.Channel.FlowOk: _flow_ok,
-        spec.Channel.Close: _close,
-        spec.Channel.CloseOk: _close_ok,
-        spec.Exchange.DeclareOk: _exchange_declare_ok,
-        spec.Exchange.DeleteOk: _exchange_delete_ok,
-        spec.Exchange.BindOk: _exchange_bind_ok,
-        spec.Exchange.UnbindOk: _exchange_unbind_ok,
-        spec.Queue.DeclareOk: _queue_declare_ok,
-        spec.Queue.BindOk: _queue_bind_ok,
-        spec.Queue.PurgeOk: _queue_purge_ok,
-        spec.Queue.DeleteOk: _queue_delete_ok,
-        spec.Queue.UnbindOk: _queue_unbind_ok,
-        spec.Basic.QosOk: _basic_qos_ok,
-        spec.Basic.ConsumeOk: _basic_consume_ok,
-        spec.Basic.Cancel: _basic_cancel_notify,
-        spec.Basic.CancelOk: _basic_cancel_ok,
-        spec.Basic.Return: _basic_return,
-        spec.Basic.Deliver: _basic_deliver,
-        spec.Basic.GetOk: _basic_get_ok,
-        spec.Basic.GetEmpty: _basic_get_empty,
-        spec.Basic.Ack: _basic_ack_recv,
-        spec.Basic.RecoverOk: _basic_recover_ok,
-        spec.Confirm.SelectOk: _confirm_select_ok,
-        spec.Tx.SelectOk: _tx_select_ok,
-        spec.Tx.CommitOk: _tx_commit_ok,
-        spec.Tx.RollbackOk: _tx_rollback_ok,
+        spec.Channel.OpenOk: _cb_open_ok,
+        spec.Channel.Flow: _cb_flow,
+        spec.Channel.FlowOk: _cb_flow_ok,
+        spec.Channel.Close: _cb_close,
+        spec.Channel.CloseOk: _cb_close_ok,
+        spec.Exchange.DeclareOk: _cb_exchange_declare_ok,
+        spec.Exchange.DeleteOk: _cb_exchange_delete_ok,
+        spec.Exchange.BindOk: _cb_exchange_bind_ok,
+        spec.Exchange.UnbindOk: _cb_exchange_unbind_ok,
+        spec.Queue.DeclareOk: _cb_queue_declare_ok,
+        spec.Queue.BindOk: _cb_queue_bind_ok,
+        spec.Queue.PurgeOk: _cb_queue_purge_ok,
+        spec.Queue.DeleteOk: _cb_queue_delete_ok,
+        spec.Queue.UnbindOk: _cb_queue_unbind_ok,
+        spec.Basic.QosOk: _cb_basic_qos_ok,
+        spec.Basic.ConsumeOk: _cb_basic_consume_ok,
+        spec.Basic.Cancel: _cb_basic_cancel_notify,
+        spec.Basic.CancelOk: _cb_basic_cancel_ok,
+        spec.Basic.Return: _cb_basic_return,
+        spec.Basic.Deliver: _cb_basic_deliver,
+        spec.Basic.GetOk: _cb_basic_get_ok,
+        spec.Basic.GetEmpty: _cb_basic_get_empty,
+        spec.Basic.Ack: _cb_basic_ack_recv,
+        spec.Basic.RecoverOk: _cb_basic_recover_ok,
+        spec.Confirm.SelectOk: _cb_confirm_select_ok,
+        spec.Tx.SelectOk: _cb_tx_select_ok,
+        spec.Tx.CommitOk: _cb_tx_commit_ok,
+        spec.Tx.RollbackOk: _cb_tx_rollback_ok,
     }
