@@ -10,7 +10,7 @@ from .serialization import AMQPWriter
 from . import __version__
 from .abstract_channel import AbstractChannel
 from .channel import Channel
-from .exceptions import ResourceError, AMQPConnectionError, error_for_code, AMQPNotImplementedError
+from .exceptions import ResourceError, AMQPConnectionError, error_for_code
 from .transport import create_transport
 from . import spec
 from .spec import Method, Frame, FrameType, method_t
@@ -63,16 +63,20 @@ class Connection(AbstractChannel):
         :param str password: password
         :param str virtual_host: virtual host
         :param str locale: locale
-        :param dict client_properties: dict of client properties
+        :param client_properties: dict of client properties
         :param ssl: dict of SSL options passed to :func:`ssl.wrap_socket()`, None to disable SSL
         :param float connect_timeout: connect timeout
         :param int channel_max: maximum number of channels
         :param int frame_max: maximum frame payload size in bytes
         :param float heartbeat: heartbeat interval in seconds, 0 disables heartbeat
-        :param Callable on_blocked: callback on connection blocked
-        :param Callable on_unblocked: callback on connection unblocked
+        :param on_blocked: callback on connection blocked
+        :param on_unblocked: callback on connection unblocked
         :param bool enable_publisher_ack: enable publisher acknowledgements by default for new channels
+        :type connect_timeout: float or None
+        :type client_properties: dict or None
         :type ssl: dict or None
+        :type on_blocked: Callable or None
+        :type on_unblocked: Callable or None
         """
         # `channels` map stores references to all active channels
         self.channels = {}  # dict of {channel_id int: Channel}
@@ -116,14 +120,15 @@ class Connection(AbstractChannel):
         login_response = login_response.getvalue()[4:]  # skip the length
 
         # reply with 'start-ok' and connection parameters
+        # noinspection PyArgumentList
         client_props = dict(LIBRARY_PROPERTIES, **client_properties or {})
-        self._x_start_ok(client_props, login_method, login_response, locale)
+        self._send_start_ok(client_props, login_method, login_response, locale)
 
         self._wait_tune_ok = True
         while self._wait_tune_ok:
             self.wait(allowed_methods=[spec.Connection.Secure, spec.Connection.Tune])
 
-        self._x_open(virtual_host)
+        self._send_open(virtual_host)
 
     @property
     def connected(self):
@@ -199,23 +204,8 @@ class Connection(AbstractChannel):
         :raise amqpy.exceptions.Timeout: if the operation times out
         """
         chan_id, method = self._wait_multiple(self.channels, None, timeout=timeout)
-        content = method.content
-
         channel = self.channels[chan_id]
-
-        if content and channel.auto_decode and hasattr(content, 'content_encoding'):
-            # noinspection PyBroadException
-            try:
-                content.body = content.body.decode(content.content_encoding)
-            except Exception:
-                pass
-
-        callback = channel.METHOD_MAP.get(method.method_type)
-
-        if callback is None:
-            raise AMQPNotImplementedError('Unknown AMQP method {} (ch: {})'.format(method.method_type, chan_id))
-
-        return callback(channel, method)
+        return self.handle_method(method, channel=channel)
 
     def close(self, reply_code=0, reply_text='', method_type=method_t(0, 0)):
         """Request a connection close
@@ -243,7 +233,7 @@ class Connection(AbstractChannel):
         self._send_method(Method(spec.Connection.Close, args))
         return self.wait(allowed_methods=[spec.Connection.Close, spec.Connection.CloseOk])
 
-    def _close(self, method):
+    def _cb_close(self, method):
         """Respond to a connection close
 
         This method indicates that the sender (server) wants to close the connection. This may be due to internal
@@ -257,24 +247,26 @@ class Connection(AbstractChannel):
         class_id = args.read_short()  # class_id of method
         method_id = args.read_short()  # method_id of method
 
-        self._x_close_ok()  # send a close-ok to the server, to confirm that we've acknowledged the close request
+        self._send_close_ok()  # send a close-ok to the server, to confirm that we've acknowledged the close request
 
         method_type = method_t(class_id, method_id)
         raise error_for_code(reply_code, reply_text, method_type, AMQPConnectionError, self.channel_id)
 
-    def _blocked(self, method):
+    def _cb_blocked(self, method):
         """RabbitMQ Extension
         """
         reason = method.args.read_shortstr()
-        if self.on_blocked:
+        if callable(self.on_blocked):
+            # noinspection PyCallingNonCallable
             return self.on_blocked(reason)
 
-    def _unblocked(self, method):
+    def _cb_unblocked(self, method):
         assert method
-        if self.on_unblocked:
+        if callable(self.on_unblocked):
+            # noinspection PyCallingNonCallable
             return self.on_unblocked()
 
-    def _x_close_ok(self):
+    def _send_close_ok(self):
         """Confirm a connection close that has been requested by the server
 
         This method confirms a Connection.Close method and tells the recipient that it is safe to release resources for
@@ -284,7 +276,7 @@ class Connection(AbstractChannel):
         self._send_method(Method(spec.Connection.CloseOk))
         self._do_close()
 
-    def _close_ok(self, method):
+    def _cb_close_ok(self, method):
         """Confirm a connection close
 
         This method is called when the server send a close-ok in response to our close request. It is now safe to
@@ -293,7 +285,7 @@ class Connection(AbstractChannel):
         assert method
         self._do_close()
 
-    def _x_open(self, virtual_host, capabilities=''):
+    def _send_open(self, virtual_host, capabilities=''):
         """Open connection to virtual host
 
         This method opens a connection to a virtual host, which is a collection of resources, and acts to separate
@@ -312,7 +304,7 @@ class Connection(AbstractChannel):
         self._send_method(Method(spec.Connection.Open, args))
         return self.wait(allowed_methods=[spec.Connection.OpenOk])
 
-    def _open_ok(self, method):
+    def _cb_open_ok(self, method):
         """Signal that the connection is ready
 
         This method signals to the client that the connection is ready for use.
@@ -320,7 +312,7 @@ class Connection(AbstractChannel):
         assert method
         log.debug('Open OK')
 
-    def _secure(self, method):
+    def _cb_secure(self, method):
         """Security mechanism challenge
 
         The SASL protocol works by exchanging challenges and responses until both peers have received sufficient
@@ -334,7 +326,7 @@ class Connection(AbstractChannel):
         challenge = method.args.read_longstr()
         assert challenge
 
-    def _x_secure_ok(self, response):
+    def _send_secure_ok(self, response):
         """Security mechanism response
 
         This method attempts to authenticate, passing a block of SASL data for the security mechanism at the server
@@ -350,7 +342,7 @@ class Connection(AbstractChannel):
         args.write_longstr(response)
         self._send_method(Method(spec.Connection.SecureOk, args))
 
-    def _start(self, method):
+    def _cb_start(self, method):
         """Start connection negotiation callback
 
         This method starts the connection negotiation process by telling the client the protocol version that the server
@@ -399,7 +391,7 @@ class Connection(AbstractChannel):
         log.debug('Security mechanisms: {}'.format(self.mechanisms))
         log.debug('Locales: {}'.format(self.locales))
 
-    def _x_start_ok(self, client_properties, mechanism, response, locale):
+    def _send_start_ok(self, client_properties, mechanism, response, locale):
         """Select security mechanism and locale
 
         This method selects a SASL security mechanism. ASL uses SASL (RFC2222) to negotiate authentication and
@@ -441,7 +433,7 @@ class Connection(AbstractChannel):
         args.write_shortstr(locale)
         self._send_method(Method(spec.Connection.StartOk, args))
 
-    def _tune(self, method):
+    def _cb_tune(self, method):
         """Propose connection tuning parameters
 
         This method is the handler for receiving a "tune" method. `channel_max` and `frame_max` are set to the lower
@@ -485,12 +477,12 @@ class Connection(AbstractChannel):
         if not self.client_heartbeat:
             self.heartbeat = 0
 
-        self._x_tune_ok(self.channel_max, self.frame_max, self.heartbeat)
+        self._send_tune_ok(self.channel_max, self.frame_max, self.heartbeat)
 
     def send_heartbeat(self):
         self.transport.write_frame(Frame(FrameType.HEARTBEAT))
 
-    def _x_tune_ok(self, channel_max, frame_max, heartbeat):
+    def _send_tune_ok(self, channel_max, frame_max, heartbeat):
         """Negotiate connection tuning parameters
 
         This method sends the client's connection tuning parameters to the server. Certain fields are negotiated, others
@@ -533,12 +525,12 @@ class Connection(AbstractChannel):
         return self.server_properties.get('capabilities') or {}
 
     METHOD_MAP = {
-        spec.Connection.Start: _start,
-        spec.Connection.Secure: _secure,
-        spec.Connection.Tune: _tune,
-        spec.Connection.OpenOk: _open_ok,
-        spec.Connection.Close: _close,
-        spec.Connection.CloseOk: _close_ok,
-        spec.Connection.Blocked: _blocked,
-        spec.Connection.Unblocked: _unblocked,
+        spec.Connection.Start: _cb_start,
+        spec.Connection.Secure: _cb_secure,
+        spec.Connection.Tune: _cb_tune,
+        spec.Connection.OpenOk: _cb_open_ok,
+        spec.Connection.Close: _cb_close,
+        spec.Connection.CloseOk: _cb_close_ok,
+        spec.Connection.Blocked: _cb_blocked,
+        spec.Connection.Unblocked: _cb_unblocked,
     }
