@@ -1,10 +1,11 @@
-"""Error Handling
+"""
+AMQP uses exceptions to handle errors:
 
-AMQP uses exceptions to handle errors.
 * Any operational error (message queue not found, insufficient access rights, etc.) results in a channel exception.
 * Any structural error (invalid argument, bad sequence of methods, etc.) results in a connection exception.
 
-An exception closes the associated channel or connection, and returns a reply code and reply text to the client.
+According to the AMQP specification, an exception closes the associated channel or connection, and returns a reply code
+and reply text to the client. However, amqpy will automatically re-open the channel after a channel error.
 """
 import struct
 from collections import namedtuple
@@ -25,12 +26,10 @@ __all__ = [
 ]
 
 
-class Timeout(Exception):
+class Timeout(TimeoutError):
     """General AMQP operation timeout
     """
-
-    def __init__(self):
-        pass
+    pass
 
 
 class AMQPError(Exception):
@@ -102,70 +101,115 @@ class ConsumerCancelled(RecoverableConnectionError):
 
 
 class ContentTooLarge(RecoverableChannelError):
+    """The client attempted to transfer content larger than the server could accept at the present time. The client may
+    retry at a later time.
+    """
     code = 311
 
 
 class NoConsumers(RecoverableChannelError):
+    """When the exchange cannot deliver to a consumer when the immediate flag is set. As a result of pending data on the
+    queue or the absence of any consumers of the queue.
+    """
     code = 313
 
 
 class ConnectionForced(RecoverableConnectionError):
+    """An operator intervened to close the connection for some reason. The client may retry at some later date.
+    """
     code = 320
 
 
 class InvalidPath(IrrecoverableConnectionError):
+    """The client tried to work with an unknown virtual host.
+    """
     code = 402
 
 
 class AccessRefused(IrrecoverableChannelError):
+    """The client attempted to work with a server entity to which it has no access due to security settings.
+    """
     code = 403
 
 
 class NotFound(IrrecoverableChannelError):
+    """The client attempted to work with a server entity that does not exist.
+    """
     code = 404
 
 
 class ResourceLocked(RecoverableChannelError):
+    """The client attempted to work with a server entity to which it has no access because another client is working
+    with it.
+    """
     code = 405
 
 
 class PreconditionFailed(IrrecoverableChannelError):
+    """The client requested a method that was not allowed because some precondition failed.
+    """
     code = 406
 
 
 class FrameError(IrrecoverableConnectionError):
+    """The sender sent a malformed frame that the recipient could not decode. This strongly implies a programming error
+    in the sending peer.
+    """
     code = 501
 
 
 class FrameSyntaxError(IrrecoverableConnectionError):
+    """The sender sent a frame that contained illegal values for one or more fields. This strongly implies a programming
+    error in the sending peer.
+    """
     code = 502
 
 
 class InvalidCommand(IrrecoverableConnectionError):
+    """The client sent an invalid sequence of frames, attempting to perform an operation that was considered invalid by
+    the server. This usually implies a programming error in the client. '
+    """
     code = 503
 
 
 class ChannelNotOpen(IrrecoverableConnectionError):
+    """The client attempted to work with a channel that had not been correctly opened. This most likely indicates a
+    fault in the client layer.
+    """
     code = 504
 
 
 class UnexpectedFrame(IrrecoverableConnectionError):
+    """The peer sent a frame that was not expected, usually in the context of a content header and body. This strongly
+    indicates a fault in the peer's content processing.
+    """
     code = 505
 
 
 class ResourceError(RecoverableConnectionError):
+    """The server could not complete the method because it lacked sufficient resources. This may be due to the client
+    creating too many of some type of entity.
+    """
     code = 506
 
 
 class NotAllowed(IrrecoverableConnectionError):
+    """The client tried to work with some entity in a manner that is prohibited by the server, due to security settings
+    or by some other criteria.
+    """
     code = 530
 
 
 class AMQPNotImplementedError(IrrecoverableConnectionError):
+    """The client tried to use functionality that is not implemented in the server.
+    """
     code = 540
 
 
 class InternalError(IrrecoverableConnectionError):
+    """The server could not complete the method because of an internal error. The server may require intervention by an
+    operator in order to resume normal operations.
+    """
     code = 541
 
 
